@@ -7,6 +7,7 @@
 #include "efacompactparser.h"
 #include "logging.h"
 #include "scopedxmlstreamreader.h"
+#include "ifopt/ifoptutil.h"
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/Location>
@@ -27,6 +28,21 @@ static void parseCompactCoordinate(const QString &text, Location &loc)
     }
 }
 
+static void parseCompactIfopt(ScopedXmlStreamReader &reader, Location &loc)
+{
+    if (reader.name() == QLatin1String("pgid")) {
+        const auto id = reader.readElementText();
+        if (IfoptUtil::isValid(id)) {
+            loc.setIdentifier(QStringLiteral("ifopt"), id);
+        }
+    } else if (reader.name() == QLatin1String("gid")) {
+        const auto id = reader.readElementText();
+        if (IfoptUtil::isValid(id) && loc.identifier(QStringLiteral("ifopt")).isEmpty()) {
+            loc.setIdentifier(QStringLiteral("ifopt"), id);
+        }
+    }
+}
+
 Location EfaCompactParser::parseCompactSf(ScopedXmlStreamReader &&reader) const
 {
     Location loc;
@@ -35,8 +51,11 @@ Location EfaCompactParser::parseCompactSf(ScopedXmlStreamReader &&reader) const
         if (reader.name() == QLatin1String("n")) {
             loc.setName(reader.readElementText());
         } else if (reader.name() == QLatin1String("ty")) {
-            if (reader.readElementText() != QLatin1String("stop")) {
-                return {};
+            const auto type = reader.readElementText();
+            if (type == QLatin1String("stop")) {
+                loc.setType(Location::Stop);
+            } else {
+                qCDebug(Log) << "unhandled location type:" << type;
             }
         } else if (reader.name() == QLatin1String("r")) {
             auto sub = reader.subReader();
@@ -44,9 +63,14 @@ Location EfaCompactParser::parseCompactSf(ScopedXmlStreamReader &&reader) const
                 if (sub.name() == QLatin1String("c")) {
                     parseCompactCoordinate(sub.readElementText(), loc);
                 } else if (sub.name() == QLatin1String("id")) {
-                    loc.setIdentifier(m_locationIdentifierType, sub.readElementText());
+                    const auto id = sub.readElementText();
+                    if (!id.isEmpty() && !isDummyStopId(id)) {
+                        loc.setIdentifier(m_locationIdentifierType, id);
+                    }
                 } else if (sub.name() == QLatin1String("pc")) {
                     loc.setLocality(sub.readElementText());
+                } else {
+                    parseCompactIfopt(sub, loc);
                 }
             }
         }
@@ -141,7 +165,11 @@ Stopover EfaCompactParser::parseCompactDp(ScopedXmlStreamReader &&reader) const
             auto subReader = reader.subReader();
             while (subReader.readNextSibling()) {
                 if (subReader.name() == QLatin1String("id")) {
-                    loc.setIdentifier(m_locationIdentifierType, subReader.readElementText());
+                    const auto id = subReader.readElementText();
+                    if (!id.isEmpty() && !isDummyStopId(id)) {
+                        loc.setIdentifier(m_locationIdentifierType, id);
+                        loc.setType(Location::Stop);
+                    }
                 } else if (subReader.name() == QLatin1String("pl")) {
                     dep.setScheduledPlatform(subReader.readElementText());
                 }
@@ -150,6 +178,8 @@ Stopover EfaCompactParser::parseCompactDp(ScopedXmlStreamReader &&reader) const
             parseCompactCoordinate(reader.readElementText(), loc);
         } else if (reader.name() == QLatin1String("ns")) {
             dep.setNotes(parseNotes(reader.subReader()));
+        } else {
+            parseCompactIfopt(reader, loc);
         }
     }
 
@@ -194,7 +224,11 @@ void EfaCompactParser::parseTripSectionHalf(ScopedXmlStreamReader &&reader, Jour
             ScopedXmlStreamReader subReader(reader.subReader());
             while (subReader.readNextSibling()) {
                 if (subReader.name() == QLatin1String("id")) {
-                    loc.setIdentifier(m_locationIdentifierType, subReader.readElementText());
+                    const auto id = subReader.readElementText();
+                    if (!id.isEmpty() && !isDummyStopId(id)) {
+                        loc.setIdentifier(m_locationIdentifierType, id);
+                        loc.setType(Location::Stop);
+                    }
                 } else if (subReader.name() == QLatin1String("pc")) {
                     loc.setLocality(subReader.readElementText());
                 } else if (subReader.name() == QLatin1String("pl") || subReader.name() == QLatin1String("divaPl")) {
@@ -203,8 +237,9 @@ void EfaCompactParser::parseTripSectionHalf(ScopedXmlStreamReader &&reader, Jour
                     parseCompactCoordinate(subReader.readElementText(), loc);
                 }
             }
+        } else {
+            parseCompactIfopt(reader, loc);
         }
-
     }
 
     if (isArr) {
@@ -310,6 +345,12 @@ JourneySection EfaCompactParser::parseTripSection(ScopedXmlStreamReader &&reader
                     }
                     loc.setCoordinate(coord[1].toFloat(), coord[0].toFloat());
 
+                    if (IfoptUtil::isValid(stopParams[13])) {
+                        loc.setIdentifier(QStringLiteral("ifopt"), stopParams[13]);
+                    } else if (IfoptUtil::isValid(stopParams[12])) {
+                        loc.setIdentifier(QStringLiteral("ifopt"), stopParams[12]);
+                    }
+
                     const auto dt = QDateTime::fromString(stopParams[2] + stopParams[3], QStringLiteral("yyyyMMddhhmm"));
                     if (!dt.isValid()) {
                         continue;
