@@ -6,6 +6,7 @@
 
 #include "opentripplannergraphqlbackend.h"
 #include "opentripplannerparser.h"
+#include "cache.h"
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/JourneyReply>
@@ -33,7 +34,8 @@ OpenTripPlannerGraphQLBackend::~OpenTripPlannerGraphQLBackend() = default;
 
 AbstractBackend::Capabilities OpenTripPlannerGraphQLBackend::capabilities() const
 {
-    return m_endpoint.startsWith(QLatin1String("https://")) ? Secure : NoCapability;
+    return (m_endpoint.startsWith(QLatin1String("https://")) ? Secure : NoCapability)
+        |  (m_apiVersion == QLatin1String("otp2") ? (CanQueryNextJourney | CanQueryPreviousJourney) : NoCapability);
 }
 
 bool OpenTripPlannerGraphQLBackend::needsLocationQuery(const Location &loc, AbstractBackend::QueryType type) const
@@ -79,13 +81,16 @@ bool OpenTripPlannerGraphQLBackend::queryLocation(const LocationRequest &req, Lo
             return;
         }
 
-        OpenTripPlannerParser p(backendId());
+        OpenTripPlannerParser p(backendId(), m_ifoptPrefix);
         p.setKnownRentalVehicleNetworks(m_rentalNetworks);
+        std::vector<Location> res;
         if (req.hasCoordinate()) {
-            addResult(reply, p.parseLocationsByCoordinate(gqlReply.data()));
+            res = p.parseLocationsByCoordinate(gqlReply.data());
         } else {
-            addResult(reply, p.parseLocationsByName(gqlReply.data()));
+            res = p.parseLocationsByName(gqlReply.data());
         }
+        Cache::addLocationCacheEntry(backendId(), reply->request().cacheKey(), res, {});
+        addResult(reply, std::move(res));
     });
 
     return true;
@@ -93,6 +98,10 @@ bool OpenTripPlannerGraphQLBackend::queryLocation(const LocationRequest &req, Lo
 
 bool OpenTripPlannerGraphQLBackend::queryStopover(const StopoverRequest &req, StopoverReply *reply, QNetworkAccessManager *nam) const
 {
+    if (!req.stop().hasCoordinate()) {
+        return false;
+    }
+
     auto gqlReq = graphQLRequest();
     gqlReq.setQueryFromFile(graphQLPath(QStringLiteral("departure.graphql")));
     gqlReq.setVariable(QStringLiteral("lat"), req.stop().latitude());
@@ -103,6 +112,7 @@ bool OpenTripPlannerGraphQLBackend::queryStopover(const StopoverRequest &req, St
     }
     gqlReq.setVariable(QStringLiteral("startTime"), dt.toSecsSinceEpoch());
     gqlReq.setVariable(QStringLiteral("startDateTime"), dt.toString(Qt::ISODate));
+    gqlReq.setVariable(QStringLiteral("maxResults"), req.maximumResults());
     // TODO arrival/departure selection?
 
     if (isLoggingEnabled()) {
@@ -113,7 +123,7 @@ bool OpenTripPlannerGraphQLBackend::queryStopover(const StopoverRequest &req, St
         if (gqlReply.error() != KGraphQLReply::NoError) {
             addError(reply, Reply::NetworkError, gqlReply.errorString());
         } else {
-            OpenTripPlannerParser p(backendId());
+            OpenTripPlannerParser p(backendId(), m_ifoptPrefix);
             addResult(reply, this, p.parseDepartures(gqlReply.data()));
         }
     });
@@ -123,22 +133,33 @@ bool OpenTripPlannerGraphQLBackend::queryStopover(const StopoverRequest &req, St
 
 bool OpenTripPlannerGraphQLBackend::queryJourney(const JourneyRequest &req, JourneyReply *reply, QNetworkAccessManager *nam) const
 {
+    if (!req.from().hasCoordinate() || !req.to().hasCoordinate()) {
+        return false;
+    }
+
     auto gqlReq = graphQLRequest();
     gqlReq.setQueryFromFile(graphQLPath(QStringLiteral("journey.graphql")));
     gqlReq.setVariable(QStringLiteral("fromLat"), req.from().latitude());
     gqlReq.setVariable(QStringLiteral("fromLon"), req.from().longitude());
     gqlReq.setVariable(QStringLiteral("toLat"), req.to().latitude());
     gqlReq.setVariable(QStringLiteral("toLon"), req.to().longitude());
+
     auto dt = req.dateTime();
+    const auto context = requestContext(req).value<OpenTripPlannerRequestContext>();
+    if (context.dateTime.isValid()) {
+        dt = context.dateTime;
+    }
     if (timeZone().isValid()) {
         dt = dt.toTimeZone(timeZone());
     }
+
     gqlReq.setVariable(QStringLiteral("date"), dt.toString(QStringLiteral("yyyy-MM-dd")));
     gqlReq.setVariable(QStringLiteral("time"), dt.toString(QStringLiteral("hh:mm:ss")));
     gqlReq.setVariable(QStringLiteral("dateTime"), dt.toString(Qt::ISODate));
     gqlReq.setVariable(QStringLiteral("arriveBy"), req.dateTimeMode() == JourneyRequest::Arrival);
     gqlReq.setVariable(QStringLiteral("maxResults"), req.maximumResults());
     gqlReq.setVariable(QStringLiteral("lang"), preferredLanguage());
+    // TODO set context.searchWindow?
 
     QJsonArray modes;
     QJsonObject walkMode;
@@ -169,9 +190,15 @@ bool OpenTripPlannerGraphQLBackend::queryJourney(const JourneyRequest &req, Jour
         if (gqlReply.error() != KGraphQLReply::NoError) {
             addError(reply, Reply::NetworkError, gqlReply.errorString());
         } else {
-            OpenTripPlannerParser p(backendId());
+            OpenTripPlannerParser p(backendId(), m_ifoptPrefix);
             p.setKnownRentalVehicleNetworks(m_rentalNetworks);
             addResult(reply, this, p.parseJourneys(gqlReply.data()));
+            if (p.m_nextJourneyContext.dateTime.isValid()) {
+                setNextRequestContext(reply, p.m_nextJourneyContext);
+            }
+            if (p.m_prevJourneyContext.dateTime.isValid()) {
+                setPreviousRequestContext(reply, p.m_prevJourneyContext);
+            }
         }
     });
 
