@@ -13,6 +13,7 @@
 #include "mergeutil_p.h"
 #include "rentalvehicle.h"
 #include "rentalvehicleutil_p.h"
+#include "ifopt/ifoptutil.h"
 
 #include <QDebug>
 #include <QHash>
@@ -146,11 +147,7 @@ struct {
 static QStringList splitAndNormalizeName(const QString &name)
 {
     static const QRegularExpression splitRegExp(QStringLiteral(R"([, \(\)-/\.\[\]])"));
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-    auto l = name.split(splitRegExp, QString::SkipEmptyParts);
-#else
     auto l = name.split(splitRegExp, Qt::SkipEmptyParts);
-#endif
 
     for (auto it = l.begin(); it != l.end();) {
         // ignore single-letter fragments, with the exception of the 'H' used in Denmark
@@ -235,6 +232,33 @@ static QString applyTransliterations(const QString &s)
     return res;
 }
 
+static bool isCompatibleLocationType(Location::Type lhs, Location::Type rhs)
+{
+    if (lhs == rhs) {
+        return true;
+    }
+    if (rhs != Location::Place) {
+        return false;
+    }
+    return lhs == Location::Stop;
+}
+
+static int isSameDistanceThreshold(Location::Type type)
+{
+    switch (type) {
+        case Location::Place:
+        case Location::Stop:
+        case Location::CarpoolPickupDropoff:
+            return 10; // meter
+        case Location::RentedVehicleStation:
+            return 5;
+        case Location::Equipment:
+        case Location::RentedVehicle:
+            return 3;
+    }
+    Q_UNREACHABLE();
+}
+
 bool Location::isSame(const Location &lhs, const Location &rhs)
 {
     const auto dist = Location::distance(lhs.latitude(), lhs.longitude(), rhs.latitude(), rhs.longitude());
@@ -243,11 +267,17 @@ bool Location::isSame(const Location &lhs, const Location &rhs)
         return false;
     }
     // incompatible types are also unmergable
-    if (lhs.type() != Place && rhs.type() != Place && lhs.type() != rhs.type()) {
+    if (!isCompatibleLocationType(lhs.type(), rhs.type()) || !isCompatibleLocationType(rhs.type(), lhs.type())) {
         return false;
     }
 
-    // ids
+    // ids - IFOPT takes priority here due to its special hierarchical handling, but only for stations
+    const auto lhsIfopt = lhs.identifier(QStringLiteral("ifopt"));
+    const auto rhsIfopt = rhs.identifier(QStringLiteral("ifopt"));
+    if (!lhsIfopt.isEmpty() && !rhsIfopt.isEmpty() && (lhs.type() == Location::Stop || rhs.type() == Location::Stop)) {
+        return IfoptUtil::isSameStopPlace(lhsIfopt, rhsIfopt);
+    }
+
     const auto lhsIds = lhs.identifiers();
     bool foundEqualId = false;
     for (auto it = lhsIds.constBegin(); it != lhsIds.constEnd(); ++it) {
@@ -269,6 +299,9 @@ bool Location::isSame(const Location &lhs, const Location &rhs)
         && !RentalVehicleStation::isSame(lhs.rentalVehicleStation(), rhs.rentalVehicleStation())) {
         return false;
     }
+    if (lhs.type() == Location::Equipment && lhs.equipment().type() != rhs.equipment().type()) {
+        return false;
+    }
 
     // name
     if (isSameName(lhs.name(), rhs.name())) {
@@ -277,8 +310,8 @@ bool Location::isSame(const Location &lhs, const Location &rhs)
 
     // TODO consider the address properties here?
 
-    // anything less than 10m apart is assumed to be the same
-    if (lhs.hasCoordinate() && rhs.hasCoordinate() && dist < 10) {
+    // anything sufficiently close together is assumed to be the same
+    if (lhs.hasCoordinate() && rhs.hasCoordinate() && dist < std::min(isSameDistanceThreshold(lhs.type()), isSameDistanceThreshold(rhs.type()))) {
         return true;
     }
 
@@ -340,6 +373,10 @@ Location Location::merge(const Location &lhs, const Location &rhs)
     // merge identifiers
     const auto rhsIds = rhs.identifiers();
     for (auto it = rhsIds.constBegin(); it != rhsIds.constEnd(); ++it) {
+        if (it.key() == QLatin1String("ifopt")) {
+            l.setIdentifier(QStringLiteral("ifopt"), IfoptUtil::merge(l.identifier(QStringLiteral("ifopt")), it.value()).toString());
+            continue;
+        }
         if (lhs.identifier(it.key()).isEmpty()) {
             l.setIdentifier(it.key(), it.value());
         }
@@ -366,6 +403,7 @@ Location Location::merge(const Location &lhs, const Location &rhs)
 
     switch (l.type()) {
         case Place:
+        case CarpoolPickupDropoff:
         case Stop:
             break;
         case RentedVehicleStation:
@@ -423,6 +461,7 @@ QJsonObject Location::toJson(const Location &loc)
             obj.remove(QLatin1String("type"));
             [[fallthrough]];
         case Stop:
+        case CarpoolPickupDropoff:
             break;
         case RentedVehicleStation:
             obj.insert(QStringLiteral("rentalVehicleStation"), RentalVehicleStation::toJson(loc.rentalVehicleStation()));
@@ -459,6 +498,7 @@ Location Location::fromJson(const QJsonObject &obj)
     switch (loc.type()) {
         case Place:
         case Stop:
+        case CarpoolPickupDropoff:
             break;
         case RentedVehicleStation:
             loc.setData(RentalVehicleStation::fromJson(obj.value(QLatin1String("rentalVehicleStation")).toObject()));
